'use strict';

/**
 * Act v2.0.1
 * @description Compact event emitter for browser (and server), with all necessary features.
 * @author BatenkovT
 */

/**
 * Returns new object with event emitter methods.
 *
 * @module Act
 * @param {Object} store Object with predefined store
 */
var Act = function Act() {
    var store = {};

    /**
     * Add handler to event, executed when this event is fired.
     *
     * @method on
     * @param {String} event The event name, or `"*" (for all store) to subscribe
     * @param {Function} handler The function to call when new event is triggered
     * @param {Boolean} [prepend=false] Add a handler to the beginning or end of the array of handlers
     */
    var on = function on(event, handler, prepend) {
        store[event] = store[event] || [];

        prepend ? store[event].unshift(handler) : store[event].push(handler);
    };

    /**
     * Add handler to event, executed only once.
     *
     * @method once
     * @param {String} event The event name, or `"*" (for all store) to subscribe
     * @param {Function} handler The function to call when new event is triggered
     * @param {Boolean} prepend Add a handler to the beginning or end of the array of handlers
     */
    var once = function once(event, handler, prepend) {
        handler._once = true;
        on(event, handler, prepend);
    };

    /**
     * Remove handler from event, event or all events.
     *
     * @method off
     * @param {String} event The event name, which handlers or event itself to remove
     * @param {Function} handler The handler to remove
     */
    var off = function off(event, handler) {
        event && store[event] ? handler ? store[event].splice(store[event].indexOf(handler), 1) : delete store[event] : store = {};
    };

    /**
     * Emit event and invoke all '*' event handlers with arguments.
     *
     * @method fire
     * @param {String} event The event name which handlers to call
     * @param {Array} args Arguments to pass to handlers
     */
    var fire = function fire(event) {
        for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            args[_key - 1] = arguments[_key];
        }

        [event, '*'].map(function (event) {
            (store[event] || []).map(function (handler) {
                handler.apply(null, args);
                handler._once && off(event, handler);
            });
        });
    };

    // return all functions as object methods
    return {

        /**
         * Getter for events object.
         *
         * @attribute events
         * @readOnly
         * @default {}
         * @type Object
         */
        get events() {
            var clone = {};

            for (var event in store) {
                clone[event] = store[event];
            }return clone;
        },
        on: on,
        once: once,
        off: off,
        fire: fire
    };
};

module.exports = Act;
