const Act = require('act.js');

describe('API methods', () => {
    const ee = Act();

    describe('property "events"', () => {
        it('should return events object', () => {
            expect(ee.events).toBeInstanceOf(Object);
        });
    });

    describe('method "on"', () => {
        it('should add handler to the end of the handlers array', () => {
            const handler = jest.fn();
            ee.on('testEvent', () => {});
            ee.on('testEvent', handler);

            expect(ee.events['testEvent'].indexOf(handler)).toEqual(1);
        });
        it('should add handler to the beginning of the handlers array', () => {
            const handler = jest.fn();
            ee.on('testEvent', handler, true);

            expect(ee.events['testEvent'].indexOf(handler)).toEqual(0);
        });
    });

    describe('method "fire"', () => {
        it('should call handlers subscribed for event', () => {
            const handler = jest.fn();
            ee.on('testEvent', handler);
            ee.fire('testEvent');

            expect(handler).toBeCalled();
        });

        it('should call handlers with arguments', () => {
            const handler = jest.fn();
            ee.on('testEvent', handler);
            ee.fire('testEvent', 'test');

            expect(handler).toHaveBeenLastCalledWith('test');
        });

        it('should call handlers subscribed for all events', () => {
            const handler = jest.fn();
            ee.on('*', handler);
            ee.on('testEvent', () => {});
            ee.fire('testEvent');

            expect(handler).toBeCalled();
        });
    });

    describe('method "once"', () => {
        it('should add handler, which can be called only once', () => {
            const handler = jest.fn();
            ee.once('testEvent', handler);
            ee.fire('testEvent');
            ee.fire('testEvent');
            
            expect(handler).toHaveBeenCalledTimes(1);
        });
        it('should add handler to the beginning of the handlers array', () => {
            const handler = jest.fn();
            ee.once('testEvent', handler, true);

            expect(ee.events['testEvent'].indexOf(handler)).toEqual(0);
        });
        it('should call handler from "*" event only once', () => {
            const handler = jest.fn();
            ee.on('testEvent', () => {});
            ee.once('*', handler);
            ee.fire('testEvent');
            ee.fire('testEvent');
            
            expect(handler).toHaveBeenCalledTimes(1);
        });
    });

    describe('method "off"', () => {
        it('should remove handler from event if it passed', () => {
            const handler = jest.fn();
            ee.on('testEvent', handler);
            ee.off('testEvent', handler);

            expect(ee.events['testEvent'].indexOf(handler)).toEqual(-1);
        });
        it('should remove event from events if handler not passed', () => {
            const handler = jest.fn();
            ee.on('testEvent', handler);
            ee.off('testEvent');

            expect(ee.events).not.toHaveProperty('testEvent');
        });
        it('should remove all events', () => {
            ee.on('testEvent', () => {});
            ee.off();

            expect(ee.events).toEqual({});
        });
    });
});

